# ---------------------------------------------------------------------------
# BadLinkFinder - A Plugin for Movable Type
#
# Release 0.3
# June 7, 2007
#
# Author: nlog(n)
# Description: http://nlogn.ath.cx/plugins/badlinkfinder/
# Email: n [at] nlogn [dot] ath [dot] cx
# ---------------------------------------------------------------------------

package MT::Plugin::BadLinkFinder;
use strict;
use MT::ConfigMgr;
use MT::Template::Context;
use Encode qw(encode decode);
MT::Template::Context->add_container_tag(BadLinkFinder => \&BadLinkFinder);

our $Name = 'BadLinkFinder';
our $VERSION = 0.3;
our $ExcludeURL = "";

if (MT->version_number >= 3) {
  require MT::Plugin;
  my $plugin = MT::Plugin->new({
    name => "$Name $VERSION",
    description => 'Adds a badlink class attribute to the anchor tag when its reference URL returns error status code.',
    doc_link => 'http://nlogn.ath.cx/plugins/badlinkfinder/'
  });
  MT->add_plugin($plugin);
}

sub BadLinkFinder {
  my ($ctx, $args) = @_;

  my $tokens = $ctx->stash('tokens');
  my $builder = $ctx->stash('builder');

  defined(my $out = $builder->build($ctx, $tokens))
    or return $ctx->error($builder->errstr);

  my $charset = {
       'UTF-8' => 'utf8',
       'EUC-JP' => 'euc-jp',
       'Shift_JIS' => 'shiftjis',
       'ISO-2022-JP' => 'iso-2022-jp'
     }->{MT::ConfigMgr->instance->PublishCharset} || 'utf8';

  $ExcludeURL ||= $ctx->stash('blog')->site_url;
  $out = decode($charset, $out);
  my $p = LinkChecker->new->parse($out);
  $out = encode($charset, $p->filtered_html);
}

1;

package LinkChecker;
use strict;
use HTTP::SimpleLinkChecker;
require HTML::Filter;
our @ISA = qw(HTML::Filter);

sub start {
   my($self, $tag, $attr, $attrseq, $orig) = @_;
   if ($tag eq 'a'
       && exists $attr->{href}
       && $attr->{href} =~ m|^http://| 
       && $attr->{href} !~ m|^$ExcludeURL|) {
     my $ua = HTTP::SimpleLinkChecker::user_agent();
     $ua->agent(__PACKAGE__ . "/" . $VERSION);
     $ua->parse_head(0);
     my $code = HTTP::SimpleLinkChecker::check_link($attr->{href});
     my $tmp = "<$tag";
     for (@$attrseq) {
       $tmp .= qq( $_="$attr->{$_}");
     }
     $tmp .= qq( class="badlink") if $code >= 400;
     $tmp .= ">";
     $self->output($tmp);
     return;
   }
   $self->output($orig);
}

sub output { push(@{$_[0]->{fhtml}}, $_[1]) }
sub filtered_html { join("", @{$_[0]->{fhtml}}) }

1;
